'''
Created on Oct 14, 2011

@author: maemo
'''

from datetime import date

from maegen.core import  maegen


from maegen.common import version

version.getInstance().submitRevision("$Revision: 155 $")

class Maegen(maegen.Maegen):
    '''
    Mock class for Maegen
    '''
    
    def __init__(self):
        '''
        Create a dummy database for GUI testing
        '''
        maegen.Maegen.__init__(self)
        self.database = maegen.Database()
        

    
    def load_database(self, database_file):
        '''
        Return a dully database with some individuals
        '''
        self.database = maegen.Database()
        indi1 = self.create_new_individual("Dupont", "Jean")
        indi1.occupation = "Tailor"
        indi1.gender = "male"
        indi = self.create_new_individual("Moore", "Roger")
        indi.birthDate = date(1943,5,24)
        indi.gender = "male"
        indi2 = self.create_new_individual("Louise", "Alberta")
        indi2.birthDate = date(1913,9,11)
        indi2.deathDate = date(1995,8,30)
        indi2.gender = "female"
              
        self.make_child(indi,   self.create_new_family(indi1, indi2))
        
        
        
    
    



# -*- encoding: UTF-8 -*-

#    Maegen is a genealogical application for N900. Use it on the go
#    to store genealogical data including individuals and relational
#    informations. Maegen can be used to browse collected data on the
#    device but the main goal is its capabilitie to export the dtabase
#    in a GEDCOM file which can be imported into any desktop genealocial
#    application.
#
#    Copyright (C) 2011  Thierry Bressure
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
'''
Created on Nov 2, 2011

@author: maemo
'''

from string import Template

from ..common import version 

version.getInstance().submitRevision("$Revision: 155 $")

class GedcomWriter():
    '''
    This class can export a maegen database into GEDCOM format 
    '''
    def __init__(self, database):
        '''
        Create a writer that can export a maegen database into GEDCOM format
        Parameter:
            - database : the maegen database
        '''
        self.database = database
        self.individual_count = 0
        self.family_count = 0
        self.individual_dic = {}
        self.family_dic = {}


    
    
    
    def export(self):
        '''
        Return a GEDCOM string for the current database
        '''
        self._give_id_to_individuals()
        self._give_id_to_families()
        return self._create_header() + self._create_submitter() + self._create_records() + self._create_trlr()


        

    def _give_id_to_individuals(self):
        for indi in self.database.individuals:
            self.individual_count += 1
            self.individual_dic[id(indi)] = self.individual_count
            
    def _give_id_to_families(self):
        for fam in self.database.families:
            self.family_count += 1
            self.family_dic[id(fam)] = self.family_count 
    
        
    def _create_header(self):
        '''
        Return the header for the current database
        '''
        header = \
"""0 HEAD
1 SOUR Maegen
2 VERS $version_of_maegen
1 SUBM @SUBMITTER@ 
1 GEDC
2 VERS 5.5
2 FORM LINEAGE-LINKED    
1 CHAR ASCII  
"""        
        return Template(header).substitute(version_of_maegen="1.0")
    
    def _create_submitter(self):
        submitter = \
"""0 @SUBMITTER@ SUBM
1 NAME $submitter
""" 
        # TODO replace maemo by the user name    
        return Template(submitter).substitute(submitter="maemo")



    def _create_personal_name_structure(self, indi, indent=1):
        """
        Return the GEDCOM part for the name of given individual
        """
        resu = ""
        name_personal = ""
        if indi.firstname :
            name_personal += indi.firstname
        else:
            name_personal += "?"
        if indi.name:
            name_personal += " /" + indi.name + "/"
        else:
            name_personal += " /?"
        resu += str(indent) + " NAME " + name_personal + "\n"
        if indi.nickname:
            resu += str(indent + 1) + NICK + indi.nickname + "\n"
        return resu
    
    
    def _convert_date_for_geneweb(self, date):
        month = None
        if date.month == 1:
            month = "JAN"
        elif date.month == 2:
            month = "FEB"
        elif date.month == 3:
            month = "MAR"
        elif date.month == 4:
            month = "APR"
        elif date.month == 5:
            month = "MAY"
        elif date.month == 6:
            month = "JUN"
        elif date.month == 7:
            month = "JUL"
        elif date.month == 8:
            month = "AUG"
        elif date.month == 9:
            month = "SEP"
        elif date.month == 10:
            month = "OCT"
        elif date.month == 11:
            month = "NOV"
        elif date.month == 12:
            month = "DEC"
            
        return  Template("$day $month $year").substitute(day=str(date.day), month=month, year=date.year)
    

    def _create_date_event_detail(self, date, indent):
        return str(indent) + " DATE " + self._convert_date_for_geneweb(date) + "\n"    
    
    def _create_birth_event(self, indi, indent=1):
        """
        Return the birth event for given individual
        """
        resu = ""
        resu += str(indent) + " BIRT\n"
        resu += self._create_date_event_detail(indi.birthDate, indent+1)
        
        return resu
    
    def _create_death_event(self, indi, indent=1):
        """
        Return the death event for given individual
        """
        resu = ""
        resu += str(indent) + " DEAT\n"
        resu += self._create_date_event_detail(indi.deathDate, indent+1)
        
        return resu

    def _create_occupation_event(self, indi, indent=1):
        """
        Return the occupation event for given individual
        """
        return str(indent) + " OCCU " + indi.occupation + "\n"
        
        
        return resu

    def _create_individual_event_structures(self, indi, indent=1):   
        resu = "" 
        if indi.birthDate:
            resu += self._create_birth_event(indi, indent)
        if indi.deathDate:
            resu+= self._create_death_event(indi, indent)

        return resu
    
    def _create_individual_attribute_structures(self, indi, indent=1):   
        resu = "" 
        if indi.occupation:
            resu += self._create_occupation_event(indi, indent)


        return resu    
    

    
    def _retrieve_individual_xref(self, individual):
        return "I" + str(self.individual_dic[id(individual)])
    
        
    def _retrieve_family_xref(self, family):
        return "F" + str(self.family_dic[id(family)])
        
    
    
    def _create_child_to_family_link(self, indi, indent=1):
        """
        Return GEDCOM line for family child link
        """
        resu = ""
        fam_list = filter(lambda f: indi in f.children, self.database.families)
        for fam in fam_list:
            resu += str(indent) + " FAMC @" + self._retrieve_family_xref(fam) + "@\n";
        return resu
        
    def _create_spouse_to_family_link(self, indi, indent=1):
        """
        Return GEDCOM line for family spouse link
        """
        resu =  ""
        fam_list = filter(lambda f: indi == f.husband or indi == f.wife, self.database.families)
        for fam in fam_list :
            resu += str(indent) + " FAMS @" + self._retrieve_family_xref(fam) + "@\n";
        return resu        
    
    

    def _create_individuals(self, indent=0):
        """
        Return all individuals in database in GEDCOM format
        """
        resu = ""
        for indi in self.database.individuals:
            resu += str(indent) + " @" + self._retrieve_individual_xref(indi) + "@ INDI\n"
            resu += self._create_personal_name_structure(indi, indent+1)
            if indi.gender == "male":
                resu += str(indent+1) + " SEX M\n"
            elif indi.gender == "female":
                resu += str(indent+1) + " SEX F\n"
            resu += self._create_individual_event_structures(indi, indent+1)
            resu += self._create_individual_attribute_structures(indi, indent+1)
            resu += self._create_child_to_family_link(indi, indent+1)
            resu += self._create_spouse_to_family_link(indi, indent+1)

        return resu
    
    def _create_families(self, indent=0):
        """
        Return all families in database in GEDCOM format
        """
        resu = ""
        for fam in self.database.families:
            resu += str(indent) + " @" + self._retrieve_family_xref(fam) + "@ FAM\n"
            if fam.husband:
                resu += str(indent+1)+ " HUSB @" + self._retrieve_individual_xref(fam.husband) + "@\n" 
            if fam.wife:
                resu += str(indent+1)+ " WIFE @" + self._retrieve_individual_xref(fam.wife) + "@\n"
            for child in fam.children:
                resu += str(indent+1)+ " CHIL @" + self._retrieve_individual_xref(child) + "@\n" 
 

        return resu
        
        
    def _create_records(self):
        """
        Return all record in GEDCOM format
        """
        return self._create_individuals()+ self._create_families()
    
    def _create_trlr(self):
        return "0 TRLR"
    
    



# -*- encoding: UTF-8 -*-

#    Maegen is a genealogical application for N900. Use it on the go
#    to store genealogical data including individuals and relational
#    informations. Maegen can be used to browse collected data on the
#    device but the main goal is its capabilitie to export the dtabase
#    in a GEDCOM file which can be imported into any desktop genealocial
#    application.
#
#    Copyright (C) 2011  Thierry Bressure
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>

'''
Created on Oct 14, 2011

@author: maemo
'''
import gtk
import hildon
import gobject
import logging
import sys, traceback
import os.path
import webbrowser
import logging
import threading
import time
import datetime
from Queue import *

#import gdata.projecthosting.client
#import gdata.projecthosting.data
#import gdata.gauth
#import gdata.client
#import gdata.data
#import atom.http_core
#import atom.core


import mock
from maegen.core import maegen

from maegen.common import version

version.getInstance().submitRevision("$Revision: 155 $")

gtk.gdk.threads_init()

def not_yet_implemented():
    show_banner_information("not yet implemented")

def show_note_information(mess):
       parent = hildon.WindowStack.get_default().peek()
       note = hildon.hildon_note_new_information(parent,mess)                 
   
       response = gtk.Dialog.run(note)

def show_banner_information(mess):
       parent = hildon.WindowStack.get_default().peek()
       banner= hildon.hildon_banner_show_information(parent,"", mess)
       banner.set_timeout(2000)  
       
def show_about_dialog(widget, data):
       '''
       Show an information dialog about the program
       '''
       window = AboutView()
       program = hildon.Program.get_instance()     
       program.add_window(window)
       window.show_all()


def _default_exception_handler(gtk_sync = False):
        '''
        Do appropriate action when the core throw a Exception.
        Currently simply display a banner message.
        '''
        type, value,tb = sys.exc_info()             
        self.__default_exception_handler(type, value, tb, gtk_sync = gtk_sync)
                        
def __default_exception_handler(type, value, tb, gtk_sync = False):
        logging.exception("!!! UNHANDLED EXCEPTION !!!")
        if gtk_sync :
            gtk.gdk.threads_enter()
        self.show_banner_information("An exception occured, please report the bug")
        window = BugReportView(type, value, tb, self.submit_issue)
        self.init_menu(window)
        self.program.add_window(window)
        window.show_all()
        if gtk_sync : 
            gtk.gdk.threads_leave()

       
def _call_handled_method(method, *arg):
       '''
       This utility function catch error on a gentle way
       ''' 
       try:
            method(*arg)
       except :
            # this is an unkown exception
            self._default_exception_handler()

      

class MaegenGui(object):
    '''
    This is the GUI of Maegen
    '''

    _last_folder = None

    def __init__(self, mocked=False):
        '''
        Create a new application GUI
        '''
        self.program = hildon.Program.get_instance()     
        if mocked :
           self.zcore = mock.Maegen()
        else:
           self.zcore = maegen.Maegen()
           
        self.__exception_fifo__ = Queue(0)
        self.exception_signal_handler_id = gobject.timeout_add(1000, self.receive_exception_signal)

        self.init_main_view()   

    def _set_default_folder_if_needed(self):
        if not self._last_folder:
            storage = os.path.expanduser("~")
            storage = os.path.join(storage, "MyDocs")
            storage = os.path.join(storage, ".documents")
            self._last_folder = storage

 


    def _default_exception_handler(self):
        '''
        Do appropriate action when the core throw a Exception.
        Currently simply display a banner message.
        '''
        type, value,tb = sys.exc_info() 
        self.__default_exception_handler(type, value, tb)
        
    def __default_exception_handler(self, type, value, tb):
        logging.exception("!!! UNHANDLED EXCEPTION !!!")      
        self.show_banner_information("An exception occured, please report the bug")     
        window = BugReportView(type, value, tb, self.submit_issue)
        self.init_menu(window)
        self.program.add_window(window)
        window.show_all()
    

    

    def send_exception_signal(self):
        '''
        Thread other than Gtk main, must call this method to post an exception
        '''
        self.__exception_fifo__.put(sys.exc_info())
        
    def receive_exception_signal(self):
        '''
        This method take an exception from the FIFO list of exception and
        display the corresponding GUI
        '''
        try:
            type, value, tb = self.__exception_fifo__.get_nowait()
            self.__default_exception_handler(type, value, tb)
        except Empty:
            pass
        return True
            

    def _call_handled_method(self, method, *arg):
        try:
            method(*arg)
        except :
            # this is an unkown exception
            self._default_exception_handler()




    def init_main_view(self):
        '''
        create a new window for the main view of the application
        '''
       
        # display a spashscreen
        window = SplashScreenView()       
        self.init_menu(window)
        self.program.add_window(window)       
        window.connect("destroy", self.quit_application, None)
        # This call show the window and also add the window to the stack
        window.show_all()

        # TODO do some stuff here                        

    def quit_application(self, widget, data):
        gobject.source_remove(self.exception_signal_handler_id)
        gtk.main_quit()
    
    def init_menu(self, window):
        '''
        create menu for all windows that will be used
        in this program
        '''
        menu = hildon.AppMenu()
                                                                                                                                                          
        newMenuBtn = hildon.GtkButton(gtk.HILDON_SIZE_AUTO);
        newMenuBtn.set_label("New");
        newMenuBtn.connect("clicked", self.show_new_dialog, None)
        menu.append(newMenuBtn)   
        
        openMenuBtn = hildon.GtkButton(gtk.HILDON_SIZE_AUTO);
        openMenuBtn.set_label("Open");
        openMenuBtn.connect("clicked", self.show_open_dialog, None)
        menu.append(openMenuBtn) 
        
        
        aboutMenuBtn = hildon.GtkButton(gtk.HILDON_SIZE_AUTO);
        aboutMenuBtn.set_label("About");
        aboutMenuBtn.connect("clicked", show_about_dialog, None)
        menu.append(aboutMenuBtn)

        
        menu.show_all()
        window.set_app_menu(menu)  
        

        
    

        

    
   
    


       
    def show_new_dialog(self, widget, data):
       '''
       Show dialog for new genealogical database
       '''
       parent = hildon.WindowStack.get_default().peek()
       fc = gobject.new(hildon.FileChooserDialog, title="New database", action=gtk.FILE_CHOOSER_ACTION_SAVE)
       fc.set_property('show-files',True)    
       fc.set_do_overwrite_confirmation(True)
       self._set_default_folder_if_needed()                   
       fc.set_current_folder(self._last_folder)
       if fc.run()==gtk.RESPONSE_OK: 
            filepath = fc.get_filename()    
            self._last_folder = fc.get_current_folder()              
            self.zcore.create_new_database(filepath)            
            window = DefaultView(self.zcore, filepath)        
            self.program.add_window(window)
            window.show_all()                     
       fc.destroy()

       
    def show_open_dialog(self, widget, data):
       '''
       Show dialog to select and open en existing genealogical database
       '''
       parent = hildon.WindowStack.get_default().peek()
       fc = gobject.new(hildon.FileChooserDialog, title="Choose a database", action=gtk.FILE_CHOOSER_ACTION_OPEN)
       fc.set_property('show-files',True)
       self._set_default_folder_if_needed()                   
       fc.set_current_folder(self._last_folder)
       if fc.run()==gtk.RESPONSE_OK: 
            filepath = fc.get_filename()
            self._last_folder = fc.get_current_folder()            
            fc.destroy()   
            self.zcore.load_database(filepath)                         
            window = DefaultView(self.zcore, filepath)        
            self.program.add_window(window)
            window.show_all()     
       else:                 
           fc.destroy()
              
    def submit_issue(self, title, description):
        # check credential
        try:
            parent = hildon.WindowStack.get_default().peek()
            dialog = hildon.LoginDialog(parent)
            dialog.set_message("Gmail account required")            
            response = dialog.run()
            username = dialog.get_username()
            password = dialog.get_password()
            dialog.destroy()
            if response == gtk.RESPONSE_OK:
                try:
                    issues_client = gdata.projecthosting.client.ProjectHostingClient()
                    issues_client.client_login(username, password,"maegen", "code")
                    versionInstance = version.getInstance()
                    versionStr = versionInstance.getVersion()
                    revisionStr = versionInstance.getRevision()
                    labels = ['Type-Defect', 'Priority-High', 'Version-' + versionStr, 'Revision-' + revisionStr]
                    issues_client.add_issue("maegen", title, description, "tbressure", labels=labels)
                except:                    
                    self.show_banner_information("failed to send issue")
                    logging.exception("Failed to report the previous issue due to")
                else:
                    self.show_banner_information("issue sent")
            else:
                self.show_banner_information("bug report cancelled")
        finally:
            hildon.WindowStack.get_default().pop_1()

    
    def run(self):
        # the splash screen  is displayed,now show the home screen in a fancy tansition               
        gtk.main()


class MaegenStackableWindow(hildon.StackableWindow):
    '''
    general design of any GUI of this app
    '''
    
    centerview = None;
    bottomButtons = None;
    
    def __init__(self, title="Maegen"):
        hildon.StackableWindow.__init__(self)
        
        self.program = hildon.Program.get_instance() 
        self.set_title(title)
   
        container = gtk.VBox();                
                  
        self.centerview = gtk.VBox() 
        
        self.init_center_view(self.centerview)
        
        pannable_area = hildon.PannableArea()
        pannable_area.set_property('mov_mode',hildon.MOVEMENT_MODE_BOTH)
        pannable_area.add_with_viewport(self.centerview)
        
        self.bottomButtons = gtk.HBox(homogeneous=True)
        
        self.init_bottom_button(self.bottomButtons)
        

        bottomAlign = gtk.Alignment(0.5,0,0,0)
        bottomAlign.add(self.bottomButtons)
        
        container.pack_start(pannable_area)
        container.pack_start(bottomAlign, expand=False)
        
        self.add(container)

    def init_center_view(self, centerview):
        '''
        This hook should be implemented by subclass to
        add widget in the centerview which is both instance
        and attribute variable for convenience
        '''
        pass;
    
    def init_bottom_button(self, bottomButtons):
        '''
        This hook should be implemented by subclass to
        add button in the bottom button which is both instance
        and attribute variable for convenience
        '''
        pass;

    def add_button(self, button):
        '''
        goodies to add a button to this view
        '''
        self.bottomButtons.pack_start(button,True,True,0)
    
    def create_button(self, name, value=None):
        '''
        goodies to create a button for this view
        ''' 
        return hildon.Button(gtk.HILDON_SIZE_THUMB_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL, name, value)

    def justifyLeft(self, widget):
          leftAlign = gtk.Alignment(0,0.5,0,0)
          leftAlign.add(widget)
          return leftAlign
      
      
class DefaultView(MaegenStackableWindow):
    '''
    This pane show somme statiscial information about the database
    and main action such as add a new individual and add a new family
    '''
    def __init__(self, zcore, database_filename):
        self.zcore = zcore       
        self.database_filename = database_filename
        self.individual_count_label = None
        self.family_count_label = None
        self.branche_count_label = None
        self.name_count_label = None
        MaegenStackableWindow.__init__(self, "Database content")
        
        # add app menu
        
        menu = hildon.AppMenu()
                                                                                                                                                          
        newMenuBtn = hildon.GtkButton(gtk.HILDON_SIZE_AUTO);
        newMenuBtn.set_label("Browse");
        newMenuBtn.connect("clicked", self.on_browse_menu_clicked, None)
        menu.append(newMenuBtn)   
        
        openMenuBtn = hildon.GtkButton(gtk.HILDON_SIZE_AUTO);
        openMenuBtn.set_label("Search");
        openMenuBtn.connect("clicked", self.on_search_menu_clicked, None)
        menu.append(openMenuBtn) 
        
        exportMenuBtn = hildon.GtkButton(gtk.HILDON_SIZE_AUTO);
        exportMenuBtn.set_label("Export");
        exportMenuBtn.connect("clicked", self.on_export_menu_clicked, None)
        menu.append(exportMenuBtn) 
        
        aboutMenuBtn = hildon.GtkButton(gtk.HILDON_SIZE_AUTO);
        aboutMenuBtn.set_label("About");
        aboutMenuBtn.connect("clicked", show_about_dialog, None)
        menu.append(aboutMenuBtn)
        
        menu.show_all()
        self.set_app_menu(menu)  
        
        # local variable
        self.selected_individual = None



    def refresh(self):
        self.individual_count_label.set_text(str(self.zcore.individuals_count()))
        self.family_count_label.set_text(str(self.zcore.families_count()))
        self.branche_count_label.set_text(str(self.zcore.branches_count()))
        self.name_count_label.set_text(str(self.zcore.names_count()))


    def on_export_menu_clicked(self, widget, data):
        dialog = gtk.Dialog()
        dialog.set_transient_for(self)
        dialog.set_title("Gedcom for database") 
        dialog.add_button("Save...", -1000)            
        gedcom = hildon.TextView()
        gedcom_source = self.zcore.export_to_gedcom()
        buffer = gtk.TextBuffer()
        buffer.set_text(gedcom_source)        
        gedcom.set_buffer(buffer)
        gedcom.set_property('editable', False)
        
        pannable_area = hildon.PannableArea()
        pannable_area.set_property('mov_mode',hildon.MOVEMENT_MODE_BOTH)
        pannable_area.set_property('size-request-policy', hildon.SIZE_REQUEST_CHILDREN)
        pannable_area.add_with_viewport(gedcom)
        dialog.vbox.add(pannable_area)
        dialog.show_all()
        response = dialog.run()
        if response == -1000:
            dialog.destroy()
            fc = gobject.new(hildon.FileChooserDialog, title="Enter gedcom file", action=gtk.FILE_CHOOSER_ACTION_SAVE)
            fc.set_property('show-files',True)    
            fc.set_do_overwrite_confirmation(True)                                        
            if fc.run()==gtk.RESPONSE_OK: 
                filepath = fc.get_filename()                                
                self.zcore.export_to_gedcom(filepath + ".ged")            
            fc.destroy()

        dialog.destroy()


    def on_browse_menu_clicked(self, widget, data):        
        '''
        Display a selector to view an individual
        '''
        selector = hildon.TouchSelector()
        model = gtk.ListStore(str, object)
        for indi in sorted(self.zcore.retrieve_all_individuals(), key=lambda indi : indi.name + indi.firstname):
            model.append([str(indi), indi])   
        selector.append_column(model, gtk.CellRendererText(),text=0)        
        selector.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_SINGLE)
 

        dialog = hildon.PickerDialog(self) 
        dialog.set_transient_for(self) 
        dialog.set_title("Select an individual")
        dialog.set_done_label("View") 
        dialog.set_selector(selector)
        
        dialog.show_all()         
        self.selected_individual = None
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            model, iter = selector.get_selected(0)
            indi = model.get(iter,1)[0]        
            dialog.destroy()    
            window = IndividualView(self.zcore,indi, self.database_filename)
            self.program.add_window(window)
            window.show_all()
        else:
            dialog.destroy()        


           
        
    def on_search_menu_clicked(self, widget, data):
        not_yet_implemented()

    def init_center_view(self, centerview):
       line = gtk.HBox()
       line.pack_start(self.justifyLeft(gtk.Label("Individual")))
       self.individual_count_label = gtk.Label(self.zcore.individuals_count())
       line.pack_start(self.individual_count_label)
       centerview.pack_start(line)

       line = gtk.HBox()
       line.pack_start(self.justifyLeft(gtk.Label("Family")))
       self.family_count_label = gtk.Label(self.zcore.families_count())
       line.pack_start(self.family_count_label)       
       centerview.pack_start(line)
        
       line = gtk.HBox()
       line.pack_start(self.justifyLeft(gtk.Label("Branch")))
       self.branche_count_label = gtk.Label(self.zcore.branches_count())
       line.pack_start(self.branche_count_label)       
       centerview.pack_start(line)

       line = gtk.HBox()
       line.pack_start(self.justifyLeft(gtk.Label("Name")))
       self.name_count_label = gtk.Label(self.zcore.names_count())
       line.pack_start(self.name_count_label)       
       centerview.pack_start(line)

    
    
    def init_bottom_button(self, bottomButtons):
        add_indi_btn = self.create_button("Add Individual")
        add_indi_btn.connect("clicked", self.on_add_individual_clicked_event, None)
        self.add_button(add_indi_btn)
        
        add_fami_btn = self.create_button("Add Family")
        add_fami_btn.connect("clicked", self.on_add_family_clicked_event, None)
        self.add_button(add_fami_btn)
        
    def on_add_individual_clicked_event(self, widget, data):
        dialog = gtk.Dialog()
        dialog.set_transient_for(self)
        dialog.set_title("New individual")
        
        dialog.add_button("Create", gtk.RESPONSE_OK)
        firstname = hildon.Entry(gtk.HILDON_SIZE_AUTO)
        name = hildon.Entry(gtk.HILDON_SIZE_AUTO)
        nickname = hildon.Entry(gtk.HILDON_SIZE_AUTO)
        dialog.vbox.add(hildon.Caption(None,"Firstname",firstname))
        dialog.vbox.add(hildon.Caption(None,"Name",name))
        dialog.vbox.add(hildon.Caption(None,"Nickname",nickname))
        dialog.show_all()
        resu = dialog.run()
        if resu == gtk.RESPONSE_OK:            
            new_indi = self.zcore.create_new_individual(name.get_text(), firstname.get_text())
            self.zcore.save_database(self.database_filename)
            self.refresh()
            new_indi.nickname = nickname.get_text()
            dialog.destroy()
            window = IndividualView(self.zcore,new_indi, self.database_filename,edit_mode=self.zcore.settings.edit_new_individual)
            self.program.add_window(window)
            window.show_all()
        else:
            dialog.destroy()
        
    
    def on_add_family_clicked_event(self, widget, data):
        dialog = gtk.Dialog()
        dialog.set_transient_for(self)
        dialog.set_title("New Family")
        
        dialog.add_button("Create", gtk.RESPONSE_OK)
       
        selector = hildon.TouchSelector()        
        husband_model = gtk.ListStore(str, object)
        wife_model = gtk.ListStore(str, object)
        logging.debug("creating list for husband selection...")
        for indi in self.zcore.retrieve_all_individuals():
            if indi.gender in ["male", None] :
                    husband_model.append([str(indi), indi])
            elif indi.gender in ["female", None] :
                    wife_model.append([str(indi), indi])
            else:
                logging.warning("unexpected gender for " + str(parent))
                husband_model.append([str(indi), indi])
                wife_model.append([str(indi), indi])
                    
        selector.append_column(husband_model, gtk.CellRendererText(), text=0)
        selector.append_column(wife_model, gtk.CellRendererText(), text=0)
        selector.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_SINGLE)
        dialog.vbox.pack_start(hildon.Caption(None, "Spouses" ,selector),expand=True)
        dialog.show_all()
        resu = dialog.run()
        if resu == gtk.RESPONSE_OK:            
            model, iter = selector.get_selected(0)
            husband = model.get(iter,1)[0]        
            model, iter = selector.get_selected(1)
            wife = model.get(iter,1)[0]        
            dialog.destroy()
            self.zcore.create_new_family(husband, wife)
            self.zcore.save_database(self.database_filename)
            self.refresh()
        else:
            dialog.destroy()


class IndividualView(MaegenStackableWindow):
    '''
    This pane show somme statistical information about the database
    and main action such as add a new individual and add a new family
    '''
    def __init__(self, zcore, individual, database_filename, edit_mode=False):
        self.zcore = zcore
        self.database_filename = database_filename
        self.individual = individual
        self.edit_mode = edit_mode
        self.edit_father = None
        self.edit_mother = None
        MaegenStackableWindow.__init__(self, str(individual))
        
                # add app menu
        
        menu = hildon.AppMenu()
                                                                                                                                                          
        if not self.edit_mode:
            editBtn = hildon.GtkButton(gtk.HILDON_SIZE_AUTO);
            editBtn.set_label("Edit");
            editBtn.connect("clicked", self._on_edit_menu_clicked, None)
            menu.append(editBtn)   
        

        
        aboutMenuBtn = hildon.GtkButton(gtk.HILDON_SIZE_AUTO);
        aboutMenuBtn.set_label("About");
        aboutMenuBtn.connect("clicked", show_about_dialog, None)
        menu.append(aboutMenuBtn)
        
        
        
        menu.show_all()
        self.set_app_menu(menu)  

    def pop_and_show_individual(self):
        hildon.WindowStack.get_default().pop_1()
        # open the clicked parent
        window = IndividualView(self.zcore, self.individual,self.database_filename, False)
        self.program.add_window(window)
        window.show_all()


    def _on_edit_menu_clicked(self, widget, data):
        # remove  the currentview
        hildon.WindowStack.get_default().pop_1()
        # open the clicked parent
        window = IndividualView(self.zcore,self.individual, self.database_filename, True)
        self.program.add_window(window)
        window.show_all()

    def init_bottom_button(self, bottomButtons):
        logging.debug("init_bottom_button for individualView...")
        if self.edit_mode: 
           logging.debug("in edit mode")
           save_btn = self.create_button("Save")
           save_btn.connect("clicked", self.on_save_clicked_event, None)
           self.add_button(save_btn)
        
           cancel_btn = self.create_button("Cancel")
           cancel_btn.connect("clicked", self.on_cancel_clicked_event, None)
           self.add_button(cancel_btn)
        else:
            logging.debug("not in edit mode, NO bottom button")
            
    def on_save_clicked_event(self, widget, data):
                            
        if self.edit_father and self.edit_mother:
            if self.father_enabled.get_active() and self.mother_enabled.get_active():                            
                self.zcore.set_parents(self.individual, self.edit_father, self.edit_mother)
            else:
                if not self.father_enabled.get_active():
                    self.zcore.remove_father(self.individual)
                if not self.mother_enabled.get_active():             
                    self.zcore.remove_mother(self.individual)
        else:    
            if self.edit_father:
                if self.father_enabled.get_active():                
                    self.zcore.set_father(self.individual,self.edit_father)
                else:                    
                    self.zcore.remove_father(self.individual)
            elif self.individual.father: 
                if not self.father_enabled.get_active():
                    self.zcore.remove_father(self.individual)
                
            if self.edit_mother:
                if self.mother_enabled.get_active():
                    self.zcore.set_mother(self.individual,self.edit_mother)
                else:
                    self.zcore.remove_mother(self.individual)
            elif self.individual.mother:
                if not self.mother_enabled.get_active():                 
                    self.zcore.remove_mother(self.individual)
            
        self.individual.name = self.edit_name.get_text()
        self.individual.firstname = self.edit_firstname.get_text()
        self.individual.nickname = self.edit_nickname.get_text()
        
        self.individual.occupation = self.edit_occupation.get_text()
        if self.birthdate_enable.get_active():
            y,m,d = self.edit_birthdate.get_date()
            self.individual.birthDate = datetime.date(y,m+1,d) 
        else:
            self.individual.birthDate = None
        if self.deathdate_enable.get_active():
            y,m,d = self.edit_deathdate.get_date()
            self.individual.deathDate = datetime.date(y,m+1,d)
        else:
            self.individual.deathDate = None
        
        model, iter = self.edit_gender_picker.get_selector().get_selected(0)
        self.individual.gender = model.get(iter,0)[0]      
        
        # after all job done, save the database
        self.zcore.save_database(self.database_filename)
        
        self.pop_and_show_individual()
        
    def on_cancel_clicked_event(self, widget, data):
        self.pop_and_show_individual()

    def _create_parent_pane(self, individual):
        logging.debug("creating parent pane for " + str(individual))
        if self.edit_mode:
            if self.edit_father:
                logging.debug("add a parent widget for edited father " + str(self.edit_father))
                self.parent_pane.pack_start(self._create_parent_widget(self.edit_father))
            elif individual.father:
                logging.debug("add a parent widget for current father " + str(individual.father))
                self.parent_pane.pack_start(self._create_parent_widget(individual.father))                
            else:
                logging.debug("add a button to set a father")
                add_father_btn = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
                add_father_btn.set_title("Add father")               
                add_father_btn.connect("clicked", self._open_dialog_to_select_parent, "father")
                self.parent_pane.pack_start(add_father_btn)
        elif individual.father:
                logging.debug("add a widget to navigate to current father " + str(individual.father))
                self.parent_pane.pack_start(self._create_parent_widget(individual.father))
        else:
            logging.debug("no widget for father because there is no father and not in edit mode")
            
                
            
        if self.edit_mode:
            if self.edit_mother:
                logging.debug("add a parent widget for edited mother " + str(self.edit_mother))
                self.parent_pane.pack_start(self._create_parent_widget(self.edit_mother))            
            elif individual.mother:
                logging.debug("add a parent widget for current mother " + str(individual.mother))
                self.parent_pane.pack_start(self._create_parent_widget(individual.mother))
            else:
                logging.debug("add a button to set a mother")                
                add_mother_btn = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
                add_mother_btn.set_title("Add mother")                
                add_mother_btn.connect("clicked", self._open_dialog_to_select_parent, "mother")
                self.parent_pane.pack_start(add_mother_btn)
        elif individual.mother:
            self.parent_pane.pack_start(self._create_parent_widget(individual.mother))
        else:
            logging.debug("no widget for mother becaue there is no mother and not in edit mode")

    def _open_dialog_to_select_parent(self, widget, parent):
        selector = hildon.TouchSelector()
        model = gtk.ListStore(str, object)
        logging.debug("creating list for parent selection...")
        for indi in self.zcore.retrieve_all_individuals():
            if parent == "father":
                if indi.gender in ["male", None] and not indi == self.individual:
                    model.append([str(indi), indi])
            elif parent == "mother":
                if indi.gender in ["female", None] and not indi == self.individual:
                    model.append([str(indi), indi])
            else:
                logging.error("unexpected parent parameter " + str(parent))
                    
        selector.append_column(model, gtk.CellRendererText(), text=0)
        selector.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_SINGLE)
        def __enable_prent_checkbox(column , user_data):
            if parent == "father":
                self.father_enabled.set_active(True)    
            elif parent == "mother":
                self.mother_enabled.set_active(True)
            else:
                logging.error("unexpected parent parameter " + str(parent)) 
            
        selector.connect("changed", __enable_prent_checkbox)
        
        dialog = hildon.PickerDialog(self)
        dialog.set_transient_for(self)
        dialog.set_title("Select an individual")
        dialog.set_done_label("Set as parent")
        dialog.set_selector(selector)
        dialog.show_all()
        self.selected_individual = None
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            model, iter = selector.get_selected(0)
            indi = model.get(iter, 1)[0]
            dialog.destroy()
            if parent == "father":
                self.edit_father = indi
            elif parent == "mother":
                self.edit_mother = indi
            else:
                logging.error("unexpected parent parmaeter " + str(parent))
            for child in self.parent_pane.get_children():
                logging.debug("removing parent widget from parent pane...")
                self.parent_pane.remove(child)
            logging.debug("attemting to add new parent widget...")
            self._create_parent_pane(self.individual)
            self.parent_pane.show_all()
        else:
            dialog.destroy()


    def _create_gender_picker(self, individual):
        picker_button = hildon.PickerButton(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_VERTICAL)
        picker_button.set_title("gender")
        selector = hildon.TouchSelector(text=True)
        picker_button.set_selector(selector)
        selector.append_text("unknown")
        selector.append_text("male")
        selector.append_text("female")
        if individual.gender == "male":
           picker_button.set_active(1)
        elif individual.gender == "female":
           picker_button.set_active(2)
        else:
           picker_button.set_active(0)
        
        return picker_button


    def _get_gender_image(self, individual):
        if individual.gender:
            if individual.gender == "male":
                pixbuf = gtk.gdk.pixbuf_new_from_file("male.png")
                image = gtk.Image()
                image.set_from_pixbuf(pixbuf)
            elif individual.gender == "female":
                pixbuf = gtk.gdk.pixbuf_new_from_file("female.png")
                image = gtk.Image()
                image.set_from_pixbuf(pixbuf)
            else:
                image = gtk.image_new_from_stock(gtk.STOCK_INFO, gtk.ICON_SIZE_BUTTON)
        else:
            image = gtk.image_new_from_stock(gtk.STOCK_INFO, gtk.ICON_SIZE_BUTTON)
        return image


    def _create_parent_widget(self, individual):
        widget = gtk.HBox()
        button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
        # Set labels value
        datestr = ""
        if individual.birthDate:
            datestr += str(individual.birthDate.year)
        
        if individual.deathDate:
            datestr += "-" + str(individual.deathDate.year)
        
        button.set_text(str(individual), datestr)
        # Set image
        image = self._get_gender_image(individual)
          
            
        button.set_image(image)
        button.set_image_position(gtk.POS_RIGHT)
                                        
        button.connect("clicked", self._on_parent_clicked_event, individual)
        widget.pack_start(button)
        if self.edit_mode:
            
            if individual == self.individual.father and not self.edit_father:
                self.edit_father = None
            elif  individual == self.individual.mother and not self.edit_mother:
                self.edit_mother= None
            elif individual == self.edit_father:
                pass
            elif individual == self.edit_mother:
                pass
            else:
                logging.error("unexpected individual parameter " + str(individual))        
                   
            if individual == self.individual.father or individual == self.edit_father:                
                self.father_enabled = hildon.CheckButton(gtk.HILDON_SIZE_AUTO)
                self.father_enabled.set_label("enabled")
                self.father_enabled.set_active(True)
                widget.pack_start(self.father_enabled, expand=False)
            if individual == self.individual.mother or individual == self.edit_mother:                
                self.mother_enabled = hildon.CheckButton(gtk.HILDON_SIZE_AUTO)
                self.mother_enabled.set_label("enabled")
                self.mother_enabled.set_active(True)
                widget.pack_start(self.mother_enabled, expand=False)
               
        return widget
    
    
    def _create_partner_widget(self, individual):
        widget = gtk.HBox()
        button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
        # Set labels value
        datestr = ""
        if individual.birthDate:
            datestr += str(individual.birthDate.year)
        
        if individual.deathDate:
            datestr += "-" + str(individual.deathDate.year)
        
        button.set_text(str(individual), datestr)
        # Set image
        image = self._get_gender_image(individual)
          
            
        button.set_image(image)
        button.set_image_position(gtk.POS_RIGHT)
                                        
        button.connect("clicked", self._on_partner_clicked_event, individual)
        widget.pack_start(button)

               
        return widget
    
    def _create_child_widget(self, individual):
        widget = gtk.HBox()
        button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
        # Set labels value
        datestr = ""
        if individual.birthDate:
            datestr += str(individual.birthDate.year)
        
        if individual.deathDate:
            datestr += "-" + str(individual.deathDate.year)
        
        button.set_text(str(individual), datestr)
        # Set image
        image = self._get_gender_image(individual)
          
            
        button.set_image(image)
        button.set_image_position(gtk.POS_RIGHT)
                                        
        button.connect("clicked", self._on_child_clicked_event, individual)
        widget.pack_start(button)

               
        return widget
    
        
    def _on_child_clicked_event(self, widget, data):
            # remove  the currentview
            hildon.WindowStack.get_default().pop_1()
            # open the clicked parent
            window = IndividualView(self.zcore,data, self.database_filename)
            self.program.add_window(window)
            window.show_all()
    
    
    def _on_partner_clicked_event(self, widget, data):
            # remove  the currentview
            hildon.WindowStack.get_default().pop_1()
            # open the clicked parent
            window = IndividualView(self.zcore,data, self.database_filename)
            self.program.add_window(window)
            window.show_all()
    
    
    def _on_parent_clicked_event(self, widget, data):
        if self.edit_mode:
            # open a dialog an individual or create a new one
            if data == self.edit_father:
                self._open_dialog_to_select_parent(widget, "father")
            elif  data == self.edit_mother:                
                self._open_dialog_to_select_parent(widget, "mother")                
            elif data == self.individual.father and self.edit_father is None:                
                self._open_dialog_to_select_parent(widget, "father")
            elif data == self.individual.mother and self.edit_mother is None:
                self._open_dialog_to_select_parent(widget,"mother")
            else:
                logging.error("unexpected data attribute " + str(data))
           

        else:
            # remove  the currentview
            hildon.WindowStack.get_default().pop_1()
            # open the clicked parent
            window = IndividualView(self.zcore,data, self.database_filename)
            self.program.add_window(window)
            window.show_all()
    

    def create_header(self, individual):
        header = gtk.HBox()
        
        # Identification
        identification = gtk.VBox()
    
        # gender
        if self.edit_mode :
            self.edit_gender_picker = self._create_gender_picker(individual)
            identification.pack_start(self.edit_gender_picker, expand=False)
        else:
             identification.pack_start(self.justifyLeft(self._get_gender_image(individual)))
        # name
        if self.edit_mode:
            self.edit_name = hildon.Entry(gtk.HILDON_SIZE_AUTO)
            if self.individual.name:
                self.edit_name.set_text(self.individual.name)
            else:
                self.edit_name.set_placeholder("enter a name")
            self.edit_firstname = hildon.Entry(gtk.HILDON_SIZE_AUTO)
            if self.individual.firstname:
                self.edit_firstname.set_text(self.individual.firstname)
            else:
                self.edit_firstname.set_placeholder("enter the firstane")
            identification.pack_start(hildon.Caption(None,"Firstname",self.edit_firstname), expand=False)
            identification.pack_start(hildon.Caption(None,"Name",self.edit_name), expand=False) 
        else:            
            identification.pack_start(self.justifyLeft(gtk.Label(str(individual))))
        
        # nickname
        if self.edit_mode:
            self.edit_nickname = hildon.Entry(gtk.HILDON_SIZE_AUTO)
            if self.individual.nickname:
                self.edit_nickname.set_text(self.individual.nickname)
            else:
                self.edit_nickname.set_placeholder("enter a nickname")           
            identification.pack_start(hildon.Caption(None,"Nickname",self.edit_nickname), expand=False)
        else:
            if individual.nickname and individual.nickname != "":
                identification.pack_start(self.justifyLeft(gtk.Label("dit " + str(individual.nickname))))
        
        header.pack_start(identification)
        
        # Description        
        description = gtk.VBox()
        
        # occupation
        if self.edit_mode: 
            self.edit_occupation = hildon.Entry(gtk.HILDON_SIZE_AUTO)
            self.edit_occupation.set_placeholder(individual.occupation)
            description.pack_start(hildon.Caption(None,"Occupation",self.edit_occupation), expand=False)
        else:
            if individual.occupation and individual.occupation != "":            
                description.pack_start(self.justifyLeft(gtk.Label("occupation: " + individual.occupation.capitalize())))
        
        # birthdate
        if self.edit_mode:            
            hbox = gtk.HBox()
            self.edit_birthdate = hildon.hildon_date_button_new_with_year_range(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL, min_year=1000, max_year=datetime.date.today().year)
            self.edit_birthdate.set_title("birth")            
            self.birthdate_enable = hildon.CheckButton(gtk.HILDON_SIZE_AUTO)
            self.birthdate_enable.set_label("enabled")
            if individual.birthDate:
                self.edit_birthdate.set_date(individual.birthDate.year, individual.birthDate.month-1, individual.birthDate.day)
                self.birthdate_enable.set_active(True)
            def __enable_birthdate_checkbox(column , user_data):
                self.birthdate_enable.set_active(True)
            self.edit_birthdate.get_selector().connect("changed", __enable_birthdate_checkbox)
            hbox.pack_start(self.edit_birthdate, expand=False)                                                                
            hbox.pack_start(self.birthdate_enable, expand=False)            
            description.pack_start(hbox , expand=False)

        else:
            if individual.birthDate :            
                description.pack_start(self.justifyLeft(gtk.Label("born on  " + str(individual.birthDate))))
        # deathdate
        if self.edit_mode:
            hbox = gtk.HBox()
            self.edit_deathdate = hildon.hildon_date_button_new_with_year_range(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL, min_year=1000, max_year=datetime.date.today().year)
            self.edit_deathdate.set_title("dead")            
            self.deathdate_enable = hildon.CheckButton(gtk.HILDON_SIZE_AUTO)
            self.deathdate_enable.set_label("enabled")
            if individual.deathDate:
                self.edit_deathdate.set_date(individual.deathDate.year, individual.deathDate.month-1, individual.deathDate.day)
                self.deathdate_enable.set_active(True)
            def __enable_deathdate_checkbox(column , user_data):
                self.deathdate_enable.set_active(True)
            self.edit_deathdate.get_selector().connect("changed", __enable_deathdate_checkbox)                
            hbox.pack_start(self.edit_deathdate, expand=False)            
            hbox.pack_start(self.deathdate_enable, expand=False)                  
            description.pack_start(hbox , expand=False)

        else:
            if individual.deathDate :            
                description.pack_start(self.justifyLeft(gtk.Label("died on " + str(individual.deathDate))))
        # age od death
        if not self.edit_mode:
            if individual.birthDate and individual.deathDate:
                description.pack_start(self.justifyLeft(gtk.Label("Age of death " + str((individual.deathDate - individual.birthDate).days / 365) + " year(s) old")))
        
        
        header.pack_start(description)
        
        return header
    

    def create_parent_pane(self, individual):
        self.parent_pane = gtk.HBox()
        
        
        self._create_parent_pane(individual)        
        
        return self.parent_pane
    
    
    def create_families_pane(self, individual):
        families_pane = gtk.VBox()
        logging.info("looking for family of " + str(individual))
        for family in self.zcore.get_families_for(individual):
            logging.info("found a family")
            one_family_pane = gtk.HBox()
            # union with
            union = gtk.VBox()
            other = None
            if  individual == family.husband:
                other = family.wife
            else:
                other = family.husband            
            union.pack_start(self.justifyLeft(gtk.Label("with")))
            if other:                                
                union.pack_start(self._create_partner_widget(other),expand=False)
            else:
                union.pack_start(self.justifyLeft(gtk.Label("unknown partner")))            
            one_family_pane.add(union)
            # children
            children = gtk.VBox()
            children.pack_start(self.justifyLeft(gtk.Label("has " + str(len(family.children)) + " child(ren)")))
            for child in family.children:
                children.pack_start(self._create_child_widget(child),expand=False)
            
            one_family_pane.add(children)
            
            families_pane.pack_start(one_family_pane)
        
        
        return families_pane
  
  

    def init_center_view(self, centerview):
        frame = gtk.Frame("Identification")
        frame.add(self.create_header(self.individual))        
        centerview.pack_start(frame, expand=False,padding=5 )
        frame = gtk.Frame("Parents")
        frame.add(self.create_parent_pane(self.individual))
        centerview.pack_start(frame,expand=False, padding=5)
        frame = gtk.Frame("Marriages and children")
        frame.add(self.create_families_pane(self.individual))
        centerview.pack_start(frame, expand=False, padding=5)

    
    
        


        

class AboutView(MaegenStackableWindow):
    '''
    This view show a fancy about dialog
    '''

    def init_center_view(self, centerview):
        
        pixbuf = gtk.gdk.pixbuf_new_from_file("maegen-logo.jpg")
        image = gtk.Image()
        image.set_from_pixbuf(pixbuf)
        centerview.add(image)
        
        centerview.add(gtk.Label("Maegen - " + version.getInstance().getVersion()))
        centerview.add(gtk.Label("Genealogical Application for N900"))
        centerview.add(gtk.Label("by Thierry Bressure - http://blog.maegen.bressure.net"))


    def init_bottom_button(self, bottomButtons):
        blogButton = self.create_button("Blog")
        blogButton.connect("clicked", self.on_blog_clicked_event, None)
        self.add_button(blogButton)
        
        siteButton = self.create_button("Site")
        siteButton.connect("clicked", self.on_site_clicked_event, None)
        self.add_button(siteButton)
        
        groupsiteButton = self.create_button("Groups")
        groupsiteButton.connect("clicked", self.on_group_clicked_event, None)
        self.add_button(groupsiteButton)
        
        licenceButton = self.create_button("Licence")
        licenceButton.connect("clicked", self.on_licence_clicked_event, None)
        self.add_button(licenceButton)

    def on_licence_clicked_event(self, widget, data):
        dialog = gtk.Dialog()
        dialog.set_transient_for(self)
        dialog.set_title("Copyright information")        
        dialog.add_button("Ok", gtk.RESPONSE_OK)
        gpl = hildon.TextView()
        gpl_licence = """
Maegen is a genealogical application for N900. Use it
on the go to store genealogical data including
individuals and relational informations. Maegen can
be used to browse collected data on the device but
the main goal is its capabilitie to export the
database in a GEDCOM file which can be imported into
any desktop genealocial application.

Copyright (C) 2011  Thierry Bressure

This program is free software: you can redistribute
it and/or modify it under the terms of the GNU
General Public License as published by the Free
Software Foundation, either version 3 of the License,
or (at your option) any later version.

This program is distributed in the hope that it will
be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public
License for more details.

You should have received a copy of the GNU General
Public License along with this program. If not, see
<http://www.gnu.org/licenses/>
        """
        buffer = gtk.TextBuffer()
        buffer.set_text(gpl_licence)        
        gpl.set_buffer(buffer)
        gpl.set_property('editable', False)
        
        pannable_area = hildon.PannableArea()
        pannable_area.set_property('mov_mode',hildon.MOVEMENT_MODE_BOTH)
        pannable_area.set_property('size-request-policy', hildon.SIZE_REQUEST_CHILDREN)
        pannable_area.add_with_viewport(gpl)
        dialog.vbox.add(pannable_area)
        dialog.show_all()
        dialog.run()
        dialog.destroy()

    def on_blog_clicked_event(self, widget, data):
         webbrowser.open_new_tab("http://blog.maegen.bressure.net");
    
    def on_site_clicked_event(self, widget, data):
         webbrowser.open_new_tab("http://maegen.bressure.net");
         
    def on_group_clicked_event(self, widget, data):
         webbrowser.open_new_tab("http://group.maegen.bressure.net");
             


class SplashScreenView(MaegenStackableWindow):
    '''
    This is the first view of the application e.g. the main view.  
    '''

    def init_center_view(self, centerview):
        pixbuf = gtk.gdk.pixbuf_new_from_file("maegen-logo.jpg")
        for i in range(1,4):
            hbox = gtk.HBox()
            for j in range(1,5):                
                image = gtk.Image()
                image.set_from_pixbuf(pixbuf)
                hbox.add(image)
            centerview.add(hbox)




       

class BugReportView(MaegenStackableWindow):
    '''
    This view show the bug and give the user the opportunity to report it
    '''
    type = None
    value = None
    traceback = None
    
    submit_issue_callback = None
    
    _body = None
    _subject = None
    
    def __init__(self, type, value, traceback, submit_issue_callback):
        self.type = type
        self.value = value
        self.traceback = traceback
        self.submit_issue_callback = submit_issue_callback
        MaegenStackableWindow.__init__(self, title="Bug reporting") 

    def init_center_view(self, centerview):
       
        subjectLbl = gtk.Label("Subject")
        centerview.pack_start(self.justifyLeft(subjectLbl), False)
        self._subject = hildon.Entry(gtk.HILDON_SIZE_FULLSCREEN_WIDTH)
        self._subject.set_placeholder("enter a subject")
        self._subject.set_text(str(self.type) + " : " + str(self.value))
        centerview.pack_start(self._subject, False)
        contentLbl = gtk.Label("Content")
        centerview.pack_start(self.justifyLeft(contentLbl), False)
        self._body = hildon.TextView()
        self._body.set_placeholder("enter the message here")
        self._body.set_wrap_mode(gtk.WRAP_WORD)
        stacktrace = traceback.format_exception(self.type, self.value, self.traceback)
        buf = self._body.get_buffer()
        for line in stacktrace:
            end = buf.get_end_iter()
            buf.insert(end, line, len(line))
        centerview.add(self._body)
        return MaegenStackableWindow.init_center_view(self, centerview)


    def init_bottom_button(self, bottomButtons):
        post = self.create_button("Post issue", None)
        post.connect("clicked", self.on_post_button_clicked, self)
        self.add_button(post)                     
        return MaegenStackableWindow.init_bottom_button(self, bottomButtons)

    def on_post_button_clicked(self, widget, view):
        buffer = view._body.get_buffer()
        body = buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter())
        subject =  view._subject.get_text()
        self.submit_issue_callback(subject, body)






# -*- encoding: UTF-8 -*-

#    Maegen is a genealogical application for N900. Use it on the go
#    to store genealogical data including individuals and relational
#    informations. Maegen can be used to browse collected data on the
#    device but the main goal is its capabilitie to export the dtabase
#    in a GEDCOM file which can be imported into any desktop genealocial
#    application.
#
#    Copyright (C) 2011  Thierry Bressure
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
'''
Created on Oct 14, 2011

@author: maemo
'''
from exceptions import  *
from gedcom import *

import logging
import pickle
import os
import os.path


from ..common import version 

version.getInstance().submitRevision("$Revision: 155 $")

def get_maegen_storage_dir():
    '''
    Compute the application storage dir.
    This is an utility function to retrieve the directory where maegen can
    store any file like settings or cached data.
    '''
    storage = os.path.expanduser("~")    
    storage = os.path.join(storage, ".maegen")
    return storage

class Maegen(object):
    '''
    Main class of the Program. The GUI use this class like a Facade to any core functions.
    '''
    
    def __init__(self):
        self._ensure_maegen_conf_store()
        settings = None
        self.database = None
        self.load_settings()
        self.apply_settings()

    def get_maegen_settings_file(self):
        storage = get_maegen_storage_dir()
        storage = os.path.join(storage, "settings.pickle")
        return storage

    def load_settings(self):
        '''
        load the saved settings
        '''
        self._ensure_maegen_conf_store()
        storage = self.get_maegen_settings_file()
        try:           
            file = open(storage,'rb')
            self.settings = pickle.load(file)
            file.close()
        except IOError, EOFError:
            logging.warning("failed to load the settings")
            self.settings  = Settings()
            
    
    def save_settings(self):
        '''
        save the current settings
        '''
        self._ensure_maegen_conf_store()
        storage = self.get_maegen_settings_file()
        try:           
            file = open(storage,'wb')
            pickle.dump(self.settings,file)
            file.close()
        except IOError:
            logging.warning("failed to save the settings")

    
    def apply_settings(self):
        pass
       
        
        
    
    def _ensure_maegen_conf_store(self):
        storage = get_maegen_storage_dir()
        if os.path.exists(storage):
            pass
        else:
            os.makedirs(storage)

        

    
    def _save_database(self, filename):
        '''
        Save the current database into a file
        '''        
        storage = filename
        try:           
            file = open(storage,'wb')
            pickle.dump(self.database,file)
            file.close()
        except IOError:
            logging.error("failed to save the database")
            raise
    
    def _load_database(self, filename):
        '''
        Read a database from a file. The new database become the current database. 
        '''                
        storage = filename
        try:           
            file = open(storage,'rb')
            self.database = pickle.load(file)
            file.close()
        except IOError, EOFError:
            logging.error("failed to load the database")
            raise 
    
    '''
    Facade function
    '''
    
    def export_to_gedcom(self, filepath=None):
        '''
        Return the gedcom for current database
        Parameter:
          - filepath the file were the gedcom is written if supplied
        '''
        gedcom_str =  GedcomWriter(self.database).export()
        if filepath:
            f = open(filepath,'w')
            try:
                f.write(gedcom_str)
            finally:
                f.close()
        return gedcom_str
    
    def get_families_for_parents(self, father, mother):
        '''
        Return all family for given father and mother
        '''
        return filter(lambda fam: fam.husband == father and fam.wife == mother, self.database.families)
    
    def get_families_for(self, individual):
        '''
        Return all family where the given individual is a parent.
        '''
        def is_parent_in(individual):
            return lambda fam: individual  in [fam.husband, fam.wife] 
            
        return filter(is_parent_in(individual), self.database.families)
    
    def get_family_with_child(self, individual):
       '''
       Return family where given individual is child.
       Raise MaegenIntegrityException if more than one family are candidate
       '''
       result = filter(lambda x: individual in x.children, self.database.families)
       if len(result) > 1:
           # since we only trace genetic filiation at most ONE family is possible       
           raise MaegenIntegrityException("individual " + individual + " is child of more than one family")
       elif len(result) == 1:
           return result[0]
       else:
           return None
   
    
    def individuals_count(self):
        '''
        Return the number of individual in the current database
        '''
        return len(self.database.individuals)
    
    def families_count(self):
        '''
        Return the number of families in the current database
        '''
        return len(self.database.families)
    
    def branches_count(self):
        '''
        Return the count of branches (tree) in the database
        '''
        logging.warning("not yet implemented")
        return 0
    
    def names_count(self):
        '''
        return the count of patronymic name in the database
        '''
        logging.warning("not yet implemented")
        return 0

    def create_new_individual(self, name="inconnu", firstname="inconnu"):
        '''
        Create a new individual and add it to the database
        Return a new individual
        '''
        resu = Individual()
        resu.name = name
        resu.firstname = firstname
        self.database.individuals.add(resu)       
        return resu
    
    def create_new_family(self, husband, wife):
        '''
        Create a new family
        '''
        resu = Family()
        resu.husband = husband
        resu.wife = wife
        self.database.families.add(resu)
        return resu
    
    def make_child(self,individual, family):
        '''
        Promote the given individual to child of given family 
        '''
        family.children.append(individual)
        individual.father = family.husband
        individual.mother = family.wife
     
    def remove_father(self, individual):
        '''
        remove the father of an individual.
        Return the removed father.
        '''    
        old = individual.father
        self.set_father(individual, None)
        return old
    
  
    def remove_mother(self, individual):
        '''
        remove the mother of an individual.
        Return the removed mother.
        '''    
        old = individual.mother
        self.set_mother(individual, None)
        return old    

    def set_parents(self, individual, father, mother):
        '''
        Make an individual children of a given father and mother
        '''    
        old_father = individual.father
        old_mother = individual.mother
        individual.fathe = father
        individual.mother = mother
        
        if old_father or old_mother:
            # ensure correctness of database 
        
            # remove from the old family
            family = self.get_family_with_child(individual)
            if family:
                if family.husband == old_father or family.wife == old_mother:
                    family.children.remove(individual)
                
        # add in a family
        fam_list = self.get_families_for_parents(father, mother)
        if len(fam_list) == 1:
            # use this family
            fam = fam_list[0]
            self.make_child(individual, fam)
        else:
            # create a new one
            new_family = self.create_new_family(father,mother)
            self.make_child(individual, new_family)

    
    def set_father(self, individual, father):
        '''
        Give a father to an individual        
        '''
        old = individual.father
        individual.father = father
        if old:
            # ensure correctness of database 
        
            # remove from the old family
            family = self.get_family_with_child(individual)
            if family:
                if family.husband == old:
                    family.children.remove(individual)
        
        # add in new family
        family = self.get_family_with_child(individual)
        if family:
            # individual is already child of a family
            if family.husband == None:
                # promote the father as husband 
                family.husband = father
            elif not family.husband == father:
                raise MaegenIntegrityException(str(individual) + " is already child in a family with an husband, cannot change husband")
        elif father:
            #create a new family with fatrher as husband
            new_family = self.create_new_family(father, None)
            self.make_child(individual, new_family)
        elif individual.mother:
            # create a family with mother as wife
            new_family = self.create_new_family(None, individual.mother)
            self.make_child(individual, new_family)
            
            
        return old

    def set_mother(self, individual, mother):
        '''
        Give a mother to an individual        
        '''
        old = individual.mother
        individual.mother = mother
        if old:
            # ensure correctness of database 
        
            # remove from the old family
            family = self.get_family_with_child(individual)
            if family:
                if family.wife == old:
                    family.children.remove(individual)
        
        # add in new family
        family = self.get_family_with_child(individual)
        if family:
            # individual is already child of a family
            if family.wife == None:
                # promote the mother as husband 
                family.wife = mother
            elif not family.husband == mother:
                raise MaegenIntegrityException(str(individual) + " is already child in a family with a wife, cannot change wife")
        elif mother:
            #create a new family with mother as wife
            new_family = self.create_new_family(None, mother)
            self.make_child(individual, new_family)
        elif individual.father:
            # create a new family with father ashusband
            new_family = self.create_new_family(individual.father, None)
            self.make_child(individual, new_family)
        return old
        
    
    def retrieve_all_individuals(self):
        '''
        return all individuals in the database as a list
        '''
        return list(self.database.individuals)
    
    def retrieve_all_families(self):
        '''
        return all families in the database as a list
        '''
        return list(self.database.families)
        

    def get_maegen_storage_dir(self):
        '''
        Storage location of megen database
        '''
        return get_maegen_storage_dir()

    
    def load_database(self, database_file):
        '''
        Read a database from a file
        '''        
        self._load_database(database_file)


    def save_database(self, database_file):
        '''
        Write current database into a file
        '''
        self._save_database(database_file)
        
    def create_new_database(self, database_file):
        '''
        Create a new empty database
        '''
        self.database = Database()
        self._save_database(database_file)
        
class Settings():
    '''
    Represents the settings for Maegen
    '''
   
    def __init__(self):
        self.edit_new_individual = False

    
        
'''
Model of genealogical data
'''

# TODO move model into a python miodule

class Database(object):
    """
    genealogical data consist in a set of individuals related by a set of families
    """
    def __init__(self):
        self.individuals = set([])
        self.families = set([])

class Individual(object):
    '''
    A person
    '''
    def __init__(self):
        self.name = ""
        self.firstname = ""
        self.nickname = ""        
        self.gender = None
        
        self.birthDate = None
        self.birthPlace = ""        
        self.deathDate = None
        self.deathPlace = ""
        
        self.note = ""
        
        self.father = None
        self.mother = None
        
        self.occupation = ""

    def __str__(self, *args, **kwrgs):
        return self.firstname.capitalize() + " " + self.name.upper()

    
    
    
class Family(object):
    '''
    A family
    '''
    
    def __init__(self):
        self.husband = None
        self.wife = None
        self.married = False
        self.divorced = False
        self.divorced_date= None
        self.married_date = None
        self.married_place = ""
        
        self.children = []
                

            